@@ -27,7 +27,7 @@
 # (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 # OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #
-
+from __future__ import print_function, absolute_import, division
 from io import BytesIO
 try:
     from StringIO import StringIO
@@ -41,7 +41,7 @@ import contextlib, weakref
 
 import llvm
 from llvm._intrinsic_ids import *
-
+from llvm.deprecated import deprecated
 from llvmpy import api
 
 #===----------------------------------------------------------------------===
@@ -211,15 +211,17 @@ class CCEnum(Enum):
     CC_X86_THISCALL  = ID.X86_ThisCall
     CC_PTX_KERNEL    = ID.PTX_Kernel
     CC_PTX_DEVICE    = ID.PTX_Device
-    CC_MBLAZE_INTR   = ID.MBLAZE_INTR
-    CC_MBLAZE_SVOL   = ID.MBLAZE_SVOL
+
+    if llvm.version <= (3, 3):
+        CC_MBLAZE_INTR   = ID.MBLAZE_INTR
+        CC_MBLAZE_SVOL   = ID.MBLAZE_SVOL
 
 CCEnum.declare()
 
 # int predicates
 class ICMPEnum(Enum):
     prefix = 'ICMP_'
-    
+
     Predicate = api.llvm.CmpInst.Predicate
 
     ICMP_EQ         = Predicate.ICMP_EQ
@@ -377,6 +379,7 @@ class Module(llvm.Wrapper):
 
     module_obj = Module.new('my_module')
     """
+    __slots__ = '__weakref__'
     __cache = weakref.WeakValueDictionary()
 
     def __new__(cls, ptr):
@@ -405,7 +408,7 @@ class Module(llvm.Wrapper):
         fileobj_or_str -- takes a file-like object or string that contains
         a module represented in bitcode.
         """
-        if isinstance(fileobj_or_str, str):
+        if isinstance(fileobj_or_str, bytes):
             bc = fileobj_or_str
         else:
             bc = fileobj_or_str.read()
@@ -449,12 +452,12 @@ class Module(llvm.Wrapper):
             """
         return str(self._ptr)
 
+    def __hash__(self):
+        return id(self._ptr)
+
     def __eq__(self, rhs):
-        assert isinstance(rhs, Module), type(rhs)
         if isinstance(rhs, Module):
-            return str(self) == str(rhs)
-        else:
-            return False
+            return self._ptr == rhs._ptr
 
     def __ne__(self, rhs):
         return not (self == rhs)
@@ -686,6 +689,7 @@ class Type(llvm.Wrapper):
     Use one of the static methods to create an instance. Example:
     ty = Type.double()
     """
+    __slots__ = '__name__'
     _type_ = api.llvm.Type
 
     def __init__(self, ptr):
@@ -878,6 +882,7 @@ class Type(llvm.Wrapper):
 
 class IntegerType(Type):
     """Represents an integer type."""
+    __slots__ = ()
     _type_ = api.llvm.IntegerType
 
     @property
@@ -887,6 +892,7 @@ class IntegerType(Type):
 
 class FunctionType(Type):
     """Represents a function type."""
+    __slots__ = ()
     _type_ = api.llvm.FunctionType
 
     @property
@@ -916,6 +922,7 @@ class FunctionType(Type):
 class StructType(Type):
     """Represents a structure type."""
     _type_ = api.llvm.StructType
+    __slots__ = ()
 
     @property
     def element_count(self):
@@ -951,7 +958,7 @@ class StructType(Type):
     def _get_name(self):
         if self._ptr.isLiteral():
            return ""
-        else:	
+        else:
            return self._ptr.getName()
 
     name = property(_get_name, _set_name)
@@ -974,6 +981,7 @@ class StructType(Type):
 class ArrayType(Type):
     """Represents an array type."""
     _type_ = api.llvm.ArrayType
+    __slots__ = ()
 
     @property
     def element(self):
@@ -985,6 +993,7 @@ class ArrayType(Type):
 
 class PointerType(Type):
     _type_ = api.llvm.PointerType
+    __slots__ = ()
 
     @property
     def pointee(self):
@@ -996,6 +1005,7 @@ class PointerType(Type):
 
 class VectorType(Type):
     _type_ = api.llvm.VectorType
+    __slots__ = ()
 
     @property
     def element(self):
@@ -1007,6 +1017,7 @@ class VectorType(Type):
 
 class Value(llvm.Wrapper):
     _type_ = api.llvm.Value
+    __slots__ = '__weakref__'
 
     def __init__(self, builder, ptr):
         assert builder is _ValueFactory
@@ -1075,6 +1086,7 @@ class Value(llvm.Wrapper):
 
 class User(Value):
     _type_ = api.llvm.User
+    __slots__ = ()
 
     @property
     def operand_count(self):
@@ -1089,6 +1101,7 @@ class User(Value):
 
 class Constant(User):
     _type_ = api.llvm.Constant
+    __slots__ = ()
 
     @staticmethod
     def null(ty):
@@ -1274,6 +1287,7 @@ class Constant(User):
 
 class ConstantExpr(Constant):
     _type_ = api.llvm.ConstantExpr
+    __slots__ = ()
 
     @property
     def opcode(self):
@@ -1284,19 +1298,20 @@ class ConstantExpr(Constant):
         return self._ptr.getOpcodeName()
 
 class ConstantAggregateZero(Constant):
-    pass
+    __slots__ = ()
 
 
 class ConstantDataArray(Constant):
-    pass
+    __slots__ = ()
 
 
 class ConstantDataVector(Constant):
-    pass
+    __slots__ = ()
 
 
 class ConstantInt(Constant):
     _type_ = api.llvm.ConstantInt
+    __slots__ = ()
 
     @property
     def z_ext_value(self):
@@ -1312,30 +1327,32 @@ class ConstantInt(Constant):
 
 
 class ConstantFP(Constant):
-    pass
+    __slots__ = ()
 
 
 class ConstantArray(Constant):
-    pass
+    __slots__ = ()
 
 
 class ConstantStruct(Constant):
-    pass
+    __slots__ = ()
 
 
 class ConstantVector(Constant):
-    pass
+    __slots__ = ()
 
 
 class ConstantPointerNull(Constant):
-    pass
+    __slots__ = ()
 
 
 class UndefValue(Constant):
-    pass
+    __slots__ = ()
+
 
 class GlobalValue(Constant):
     _type_ = api.llvm.GlobalValue
+    __slots__ = ()
 
     def _get_linkage(self):
         return self._ptr.getLinkage()
@@ -1381,6 +1398,7 @@ class GlobalValue(Constant):
 
 class GlobalVariable(GlobalValue):
     _type_ = api.llvm.GlobalVariable
+    __slots__ = ()
 
     @staticmethod
     def new(module, ty, name, addrspace=0):
@@ -1441,6 +1459,7 @@ class GlobalVariable(GlobalValue):
     thread_local = property(_get_thread_local, _set_thread_local)
 
 class Argument(Value):
+    __slots__ = ()
     _type_ = api.llvm.Argument
     _valid_attrs = frozenset([ATTR_BY_VAL, ATTR_NEST, ATTR_NO_ALIAS,
                               ATTR_NO_CAPTURE, ATTR_STRUCT_RET])
@@ -1452,7 +1471,7 @@ class Argument(Value):
             attrbldr.addAttribute(attr)
             attrs = api.llvm.AttributeSet.get(context, 0, attrbldr)
             self._ptr.addAttr(attrs)
-        
+
             if attr not in self:
                 raise ValueError("Attribute %r is not valid for arg %s" %
                                  (attr, self))
@@ -1541,6 +1560,7 @@ class Argument(Value):
         return self._ptr.hasStructRetAttr()
 
 class Function(GlobalValue):
+    __slots__ = ()
     _type_ = api.llvm.Function
 
     @staticmethod
@@ -1624,7 +1644,6 @@ class Function(GlobalValue):
 
     @property
     def entry_basic_block(self):
-        assert self.basic_block_count
         return _make_value(self._ptr.getEntryBlock())
 
     def get_entry_basic_block(self):
@@ -1650,7 +1669,10 @@ class Function(GlobalValue):
         context = api.llvm.getGlobalContext()
         attrbldr = api.llvm.AttrBuilder.new()
         attrbldr.addAttribute(attr)
-        attrs = api.llvm.Attributes.get(context, attrbldr)
+        if llvm.version >= (3, 3):
+            attrs = api.llvm.Attribute.get(context, attrbldr)
+        else:
+            attrs = api.llvm.Attributes.get(context, attrbldr)
         self._ptr.removeFnAttr(attrs)
 
     def viewCFGOnly(self):
@@ -1676,6 +1698,7 @@ class Function(GlobalValue):
 #===----------------------------------------------------------------------===
 
 class InlineAsm(Value):
+    __slots__ = ()
     _type_ = api.llvm.InlineAsm
 
     @staticmethod
@@ -1690,6 +1713,7 @@ class InlineAsm(Value):
 #===----------------------------------------------------------------------===
 
 class MetaData(Value):
+    __slots__ = ()
     _type_ = api.llvm.MDNode
 
     @staticmethod
@@ -1746,6 +1770,7 @@ class MetaDataString(Value):
 
 
 class NamedMetaData(llvm.Wrapper):
+    __slots__ = ()
 
     @staticmethod
     def get_or_insert(mod, name):
@@ -1775,6 +1800,7 @@ class NamedMetaData(llvm.Wrapper):
 #===----------------------------------------------------------------------===
 
 class Instruction(User):
+    __slots__ = ()
     _type_ = api.llvm.Instruction
 
     @property
@@ -1852,10 +1878,11 @@ class Instruction(User):
         return self._ptr.eraseFromParent()
 
     def replace_all_uses_with(self, inst):
-        self._ptr.replaceAllUsesWith(inst)
+        self._ptr.replaceAllUsesWith(inst._ptr)
 
 
 class CallOrInvokeInstruction(Instruction):
+    __slots__ = ()
     _type_ = api.llvm.CallInst, api.llvm.InvokeInst
 
     def _get_cc(self):
@@ -1870,21 +1897,33 @@ class CallOrInvokeInstruction(Instruction):
         context = api.llvm.getGlobalContext()
         attrbldr = api.llvm.AttrBuilder.new()
         attrbldr.addAttribute(attr)
-        attrs = api.llvm.Attributes.get(context, attrbldr)
+        if llvm.version >= (3, 3):
+            attrs = api.llvm.Attribute.get(context, attrbldr)
+        else:
+            attrs = api.llvm.Attributes.get(context, attrbldr)
+
         self._ptr.addAttribute(idx, attrs)
 
     def remove_parameter_attribute(self, idx, attr):
         context = api.llvm.getGlobalContext()
         attrbldr = api.llvm.AttrBuilder.new()
         attrbldr.addAttribute(attr)
-        attrs = api.llvm.Attributes.get(context, attrbldr)
+        if llvm.version >= (3, 3):
+            attrs = api.llvm.Attribute.get(context, attrbldr)
+        else:
+            attrs = api.llvm.Attributes.get(context, attrbldr)
+
         self._ptr.removeAttribute(idx, attrs)
 
     def set_parameter_alignment(self, idx, align):
         context = api.llvm.getGlobalContext()
         attrbldr = api.llvm.AttrBuilder.new()
         attrbldr.addAlignmentAttr(align)
-        attrs = api.llvm.Attributes.get(context, attrbldr)
+        if llvm.version >= (3, 3):
+            attrs = api.llvm.Attribute.get(context, attrbldr)
+        else:
+            attrs = api.llvm.Attributes.get(context, attrbldr)
+
         self._ptr.addAttribute(idx, attrs)
 
     def _get_called_function(self):
@@ -1899,6 +1938,7 @@ class CallOrInvokeInstruction(Instruction):
 
 
 class PHINode(Instruction):
+    __slots__ = ()
     _type_ = api.llvm.PHINode
 
     @property
@@ -1916,6 +1956,7 @@ class PHINode(Instruction):
 
 
 class SwitchInstruction(Instruction):
+    __slots__ = ()
     _type_ = api.llvm.SwitchInst
 
     def add_case(self, const, bblk):
@@ -1923,6 +1964,7 @@ class SwitchInstruction(Instruction):
 
 
 class CompareInstruction(Instruction):
+    __slots__ = ()
     _type_ = api.llvm.CmpInst
 
     @property
@@ -1934,11 +1976,40 @@ class CompareInstruction(Instruction):
             return FCMPEnum.get(n)
 
 
+class AllocaInstruction(Instruction):
+    __slots__ = ()
+    _type_ = api.llvm.AllocaInst
+
+    @property
+    def alignment(self):
+        return self._ptr.getAlignment()
+
+    @alignment.setter
+    def alignment(self, n):
+        self._ptr.setAlignment(n)
+
+    @property
+    def array_size(self):
+        return self._ptr.getArraySize()
+
+    @array_size.setter
+    def array_size(self, value):
+        return self._ptr.setArraySize(value._ptr)._ptr
+
+    @property
+    def is_array(self):
+        return self._ptr.isArrayAllocation()
+
+    @property
+    def is_static(self):
+        return self._ptr.isStaticAlloca()
+
 #===----------------------------------------------------------------------===
 # Basic block
 #===----------------------------------------------------------------------===
 
 class BasicBlock(Value):
+    __slots__ = ()
     _type_ = api.llvm.BasicBlock
 
     def insert_before(self, name):
@@ -1957,14 +2028,36 @@ class BasicBlock(Value):
 
     @property
     def instructions(self):
+        """
+        Returns a list of instructions.
+        Note: This function is expensive.  To access the terminator of the
+        block, use BasicBlock.terminator.
+        """
         return list(map(_make_value, self._ptr.getInstList()))
 
+    @property
+    def terminator(self):
+        """Returns None or the terminator of this basicblock.
+        """
+        inst = self._ptr.getTerminator()
+        return _make_value(inst) if inst is not None else None
+
+    @property
+    def successors(self):
+        """Returns a list of successor basic blocks.
+        """
+        if not hasattr(self.terminator._ptr, "getSuccessor"):
+            return []
+        return list(map(_make_value, map(self.terminator._ptr.getSuccessor,
+            range(0, self.terminator._ptr.getNumSuccessors()))))
+
 #===----------------------------------------------------------------------===
 # Value factory method
 #===----------------------------------------------------------------------===
 
 
 class _ValueFactory(object):
+    __slots__ = ()
     cache = weakref.WeakValueDictionary()
 
     # value ID -> class map
@@ -1993,7 +2086,8 @@ class _ValueFactory(object):
         VALUE_INSTRUCTION + OPCODE_INVOKE     : CallOrInvokeInstruction,
         VALUE_INSTRUCTION + OPCODE_SWITCH     : SwitchInstruction,
         VALUE_INSTRUCTION + OPCODE_ICMP       : CompareInstruction,
-        VALUE_INSTRUCTION + OPCODE_FCMP       : CompareInstruction
+        VALUE_INSTRUCTION + OPCODE_FCMP       : CompareInstruction,
+        VALUE_INSTRUCTION + OPCODE_ALLOCA     : AllocaInstruction,
     }
 
     @classmethod
@@ -2040,6 +2134,7 @@ _atomic_orderings = {
 }
 
 class Builder(llvm.Wrapper):
+    __slots__ = ()
 
     @staticmethod
     def new(basic_block):
@@ -2201,7 +2296,6 @@ class Builder(llvm.Wrapper):
     # memory
 
     def malloc(self, ty, name=""):
-        context = api.llvm.getGlobalContext()
         allocsz = api.llvm.ConstantExpr.getSizeOf(ty._ptr)
         ity = allocsz.getType()
         malloc = api.llvm.CallInst.CreateMalloc(self.basic_block._ptr,
@@ -2215,7 +2309,6 @@ class Builder(llvm.Wrapper):
         return _make_value(inst)
 
     def malloc_array(self, ty, size, name=""):
-        context = api.llvm.getGlobalContext()
         allocsz = api.llvm.ConstantExpr.getSizeOf(ty._ptr)
         ity = allocsz.getType()
         malloc = api.llvm.CallInst.CreateMalloc(self.basic_block._ptr,
@@ -2228,12 +2321,13 @@ class Builder(llvm.Wrapper):
         inst = self._ptr.Insert(malloc, name)
         return _make_value(inst)
 
-    def alloca(self, ty, name=""):
-        intty = Type.int()
-        return _make_value(self._ptr.CreateAlloca(ty._ptr, None, name))
+    def alloca(self, ty, size=None, name=""):
+        sizeptr = size._ptr if size else None
+        return _make_value(self._ptr.CreateAlloca(ty._ptr, sizeptr, name))
 
+    @deprecated
     def alloca_array(self, ty, size, name=""):
-        return _make_value(self._ptr.CreateAlloca(ty._ptr, size._ptr, name))
+        return self.alloca(ty, size, name=name)
 
     def free(self, ptr):
         free = api.llvm.CallInst.CreateFree(ptr._ptr, self.basic_block._ptr)
@@ -2320,15 +2414,20 @@ class Builder(llvm.Wrapper):
     # misc
 
     def extract_value(self, retval, idx, name=""):
-        return _make_value(self._ptr.CreateExtractValue(retval._ptr, [idx], name))
+        if not isinstance(idx, (tuple, list)):
+            idx = [idx]
+        return _make_value(self._ptr.CreateExtractValue(retval._ptr, idx,
+                                                        name))
 
     # obsolete synonym for extract_value
     getresult = extract_value
 
     def insert_value(self, retval, rhs, idx, name=""):
+        if not isinstance(idx, (tuple, list)):
+            idx = [idx]
         return _make_value(self._ptr.CreateInsertValue(retval._ptr,
                                                        rhs._ptr,
-                                                       [idx],
+                                                       idx,
                                                        name))
 
     def phi(self, ty, name=""):
