@@ -40,7 +40,7 @@
 
 namespace extra{
     using namespace llvm;
-    
+
     class raw_svector_ostream_helper: public raw_svector_ostream {
         SmallVectorImpl<char> *SV;
     public:
@@ -85,7 +85,7 @@ PyObject* make_raw_ostream_for_printing(PyObject* self, PyObject* args)
 {
     using extra::raw_svector_ostream_helper;
     using llvm::raw_svector_ostream;
-    
+
     if (!PyArg_ParseTuple(args, "")) {
         return NULL;
     }
@@ -99,7 +99,7 @@ PyObject* make_small_vector_from_types(PyObject* self, PyObject* args)
 {
     using llvm::Type;
     typedef llvm::SmallVector<llvm::Type*, 8> SmallVector_Type;
-    
+
     SmallVector_Type* SV = new SmallVector_Type;
     Py_ssize_t size = PyTuple_Size(args);
     for (Py_ssize_t i = 0; i < size; ++i) {
@@ -186,8 +186,8 @@ PyObject* iterator_to_pylist_deref(iterator begin, iterator end,
 {
     PyObject* list = PyList_New(0);
     for(; begin != end; ++begin) {
-        PyObject* cap = pycapsule_new(&*begin, capsuleName, className);
-        PyList_Append(list, cap);
+        auto_pyobject cap = pycapsule_new(&*begin, capsuleName, className);
+        PyList_Append(list, *cap);
     }
     return list;
 }
@@ -198,8 +198,8 @@ PyObject* iterator_to_pylist(iterator begin, iterator end,
 {
     PyObject* list = PyList_New(0);
     for(; begin != end; ++begin) {
-        PyObject* cap = pycapsule_new(*begin, capsuleName, className);
-        PyList_Append(list, cap);
+        auto_pyobject cap = pycapsule_new(*begin, capsuleName, className);
+        PyList_Append(list, *cap);
     }
     return list;
 }
@@ -420,7 +420,7 @@ PyObject* ExecutionEngine_RunFunction(llvm::ExecutionEngine* EE,
             PyErr_SetString(PyExc_RuntimeError, "Failed to index into args?");
             return NULL;
         }
-        
+
         GenericValue* gv = static_cast<GenericValue*>(
                               PyCapsule_GetPointer(obj, GVN));
 
@@ -575,11 +575,11 @@ PyObject* TargetMachine_addPassesToEmitFile(
     bool status = TM->addPassesToEmitFile(PM, fso, FTy, disableVerify);
     if (status) {
         StringRef sr = rso.str();
-        PyObject* buf = PyString_FromStringAndSize(sr.data(), sr.size());
+        auto_pyobject buf = PyString_FromStringAndSize(sr.data(), sr.size());
         if (!buf) {
             return NULL;
         }
-        if (-1 == PyFile_WriteObject(buf, Out, Py_PRINT_RAW)){
+        if (-1 == PyFile_WriteObject(*buf, Out, Py_PRINT_RAW)){
             return NULL;
         }
         Py_RETURN_TRUE;
@@ -626,7 +626,7 @@ PyObject* Linker_LinkInModule(llvm::Linker* Linker,
     if (! failed) {
         Py_RETURN_FALSE;
     } else {
-    
+
         auto_pyobject buf = PyBytes_FromString(errmsg.c_str());
         if (NULL == callwrite(ErrMsg, *buf)){
             return NULL;
@@ -861,6 +861,27 @@ PyObject* DynamicLibrary_LoadLibraryPermanently(const char * Filename,
     }
 }
 
+static
+PyObject* DynamicLibrary_getPermanentLibrary(const char * Filename,
+                                                PyObject* ErrMsg = 0)
+{
+    using namespace llvm::sys;
+    std::string errmsg;
+    DynamicLibrary dylib = DynamicLibrary::getPermanentLibrary(Filename, &errmsg);
+    if (!dylib.isValid()) {
+        if (ErrMsg) {
+            auto_pyobject buf = PyBytes_FromString(errmsg.c_str());
+            if (!callwrite(ErrMsg, *buf))
+                return NULL;
+        }
+        PyErr_SetString(PyExc_RuntimeError, errmsg.c_str());
+        return NULL;
+    }
+    return pycapsule_new(new DynamicLibrary(dylib),
+                     "llvm::sys::DynamicLibrary",
+                     "llvm::sys::DynamicLibrary");
+}
+
 class PassRegistryEnumerator : public llvm::PassRegistrationListener{
 public:
     PyObject* List;
@@ -870,7 +891,8 @@ public:
     inline virtual void passEnumerate(const llvm::PassInfo * pass_info){
         PyObject* passArg = PyString_FromString(pass_info->getPassArgument());
         PyObject* passName = PyString_FromString(pass_info->getPassName());
-        PyList_Append(List, Py_BuildValue("(OO)", passArg, passName));
+        auto_pyobject pair = Py_BuildValue("(OO)", passArg, passName);
+        PyList_Append(List, *pair);
     }
 };
 
@@ -988,7 +1010,7 @@ fail:
 }
 
 static
-PyObject* MCDisassembler_getInstruction(llvm::MCDisassembler *disasm, 
+PyObject* MCDisassembler_getInstruction(llvm::MCDisassembler *disasm,
                                         llvm::MCInst &instr,
                                         const llvm::MemoryObject &region,
                                         uint64_t address
@@ -998,7 +1020,7 @@ PyObject* MCDisassembler_getInstruction(llvm::MCDisassembler *disasm,
     llvm::MCDisassembler::DecodeStatus status;
 
     size = 0;
-    status = disasm->getInstruction(instr, size, region, address, 
+    status = disasm->getInstruction(instr, size, region, address,
                                     llvm::nulls(), llvm::nulls());
     return Py_BuildValue("(i,i)", int(status), size);
 }
@@ -1046,5 +1068,5 @@ PyObject* llvm_sys_isBigEndianHost()
     else
         Py_RETURN_FALSE;
 }
-#endif 
+#endif
 
