@@ -12,6 +12,15 @@ from src.Target.TargetMachine import TargetMachine
 from src.Target.TargetOptions import TargetOptions
 from src.Support.CodeGen import Reloc, CodeModel, CodeGenOpt
 
+if LLVM_VERSION >= (3, 4):
+    from src.MC import MCSubtargetInfo
+    from src.MC import MCDisassembler
+    from src.MC import MCRegisterInfo
+    from src.MC import MCAsmInfo
+    from src.MC import MCInstrInfo
+    from src.MC import MCInstrAnalysis
+    from src.MC import MCInstPrinter
+
 @Target
 class Target:
     getNext = Method(const(ownedptr(Target)))
@@ -43,7 +52,35 @@ class Target:
                                  CodeGenOpt.Level,  # = CodeGenOpt.Default
                                  ).require_only(4)
 
+    if LLVM_VERSION >= (3, 4):
+        createMCSubtargetInfo = Method(ptr(MCSubtargetInfo),
+                                       cast(str, StringRef), #triple
+                                       cast(str, StringRef), #cpu
+                                       cast(str, StringRef)  #features
+                                       )
 
+        createMCDisassembler = Method(ptr(MCDisassembler), ref(MCSubtargetInfo))
+
+        createMCRegInfo = Method(ptr(MCRegisterInfo),
+                                 cast(str, StringRef)        #Triple
+                                 )
+
+        createMCAsmInfo = Method(ptr(MCAsmInfo),
+                                 const(ref(MCRegisterInfo)), #MRI
+                                 cast(str, StringRef)        #Triple
+                                 )
+
+        createMCInstrInfo = Method(ptr(MCInstrInfo))
+
+        createMCInstrAnalysis = Method(ptr(MCInstrAnalysis), const(ptr(MCInstrInfo)))
+
+        createMCInstPrinter = Method(ptr(MCInstPrinter),
+                                     cast(int, Unsigned),        #SyntaxVariant
+                                     const(ref(MCAsmInfo)),      #MAI
+                                     const(ref(MCInstrInfo)),    #MII
+                                     const(ref(MCRegisterInfo)), #MRI
+                                     const(ref(MCSubtargetInfo)) #STI
+                                     )
 @TargetRegistry
 class TargetRegistry:
     printRegisteredTargetsForVersion = StaticMethod()
@@ -66,3 +103,6 @@ class TargetRegistry:
                                     PyObjectPtr,          # const Target*
                                     PyObjectPtr,          # std::string &Error
                                     )
+
+    targetsList = CustomStaticMethod('TargetRegistry_targets_list', PyObjectPtr)
+
