@@ -33,10 +33,16 @@ class PassManagerBuilder:
         return Attr(getter=cast(Bool, bool),
                     setter=cast(bool, Bool))
 
-    DisableSimplifyLibCalls = _attr_bool()
+    if LLVM_VERSION <= (3, 3):
+        DisableSimplifyLibCalls = _attr_bool()
+
     DisableUnitAtATime = _attr_bool()
     DisableUnrollLoops = _attr_bool()
-    Vectorize = _attr_bool()
+    if LLVM_VERSION >= (3, 3):
+        BBVectorize = _attr_bool()
+        SLPVectorize = _attr_bool()
+    else:
+        Vectorize = _attr_bool()
     LoopVectorize = _attr_bool()
 
     LibraryInfo = Attr(getter=ownedptr(TargetLibraryInfo),
